# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class ControlPointSelection(enum.IntEnum):
    """Control point selection type.
    """
    MANUAL = 0
    """Manual Spline control point selection."""
    PROGRAMCONTROLLED = 1
    """Program controlled spline control point selection."""

class SplineRefinementType(enum.IntEnum):
    """Type of spline refinement. Currently, supports h-refinement and p-refinement.
    """
    H = 0
    """H refinement of spline."""
    P = 1
    """P refinement of spline."""

class SplineFeatureCaptureType(enum.IntEnum):
    """Options to capture features in spline.
    """
    IGNORE = 0
    """Ignores feature capture in spline."""
    BYANGLE = 1
    """Uses angle for capturing features in spline."""
    BYEDGES = 2
    """Use edges for capturing features in spline."""

class SplineContinuityType(enum.IntEnum):
    """Spline Continuity types.
    """
    CUBICC0 = 0
    """Spline use cubic C0 continuity."""
    CUBICC1 = 1
    """Spline use cubic C1 continuity."""
    QUADRATICC0 = 2
    """Spline use quadratic C0 continuity."""

class IGAResults(CoreObject):
    """Results of IGA operations.

    Parameters
    ----------
    model: Model
        Model to create a ``IGAResults`` object with default parameters.
    error_code: ErrorCode, optional
        Error code if IGA operation is unsuccessful.
    warning_code: WarningCode, optional
        Warning code if IGA operation is partially successful.
    spline_ids: Iterable[int], optional
        Ids of the created spline.
    json_data: dict, optional
        JSON dictionary to create a ``IGAResults`` object with provided parameters.

    Examples
    --------
    >>> iga_results = prime.IGAResults(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            warning_code: WarningCode,
            spline_ids: Iterable[int]):
        self._error_code = ErrorCode(error_code)
        self._warning_code = WarningCode(warning_code)
        self._spline_ids = spline_ids if isinstance(spline_ids, np.ndarray) else np.array(spline_ids, dtype=np.int32) if spline_ids is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            warning_code: WarningCode = None,
            spline_ids: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``IGAResults`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``IGAResults`` object with default parameters.
        error_code: ErrorCode, optional
            Error code if IGA operation is unsuccessful.
        warning_code: WarningCode, optional
            Warning code if IGA operation is partially successful.
        spline_ids: Iterable[int], optional
            Ids of the created spline.
        json_data: dict, optional
            JSON dictionary to create a ``IGAResults`` object with provided parameters.

        Examples
        --------
        >>> iga_results = prime.IGAResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                WarningCode(json_data["warningCode"] if "warningCode" in json_data else None),
                json_data["splineIds"] if "splineIds" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, warning_code, spline_ids])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    warning_code,
                    spline_ids)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "IGAResults")
                    json_data = param_json["IGAResults"] if "IGAResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( IGAResults._default_params["error_code"] if "error_code" in IGAResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        warning_code if warning_code is not None else ( IGAResults._default_params["warning_code"] if "warning_code" in IGAResults._default_params else WarningCode(json_data["warningCode"] if "warningCode" in json_data else None)),
                        spline_ids if spline_ids is not None else ( IGAResults._default_params["spline_ids"] if "spline_ids" in IGAResults._default_params else (json_data["splineIds"] if "splineIds" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            warning_code: WarningCode = None,
            spline_ids: Iterable[int] = None):
        """Set the default values of the ``IGAResults`` object.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code if IGA operation is unsuccessful.
        warning_code: WarningCode, optional
            Warning code if IGA operation is partially successful.
        spline_ids: Iterable[int], optional
            Ids of the created spline.
        """
        args = locals()
        [IGAResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``IGAResults`` object.

        Examples
        --------
        >>> IGAResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in IGAResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._warning_code is not None:
            json_data["warningCode"] = self._warning_code
        if self._spline_ids is not None:
            json_data["splineIds"] = self._spline_ids
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nwarning_code :  %s\nspline_ids :  %s" % (self._error_code, self._warning_code, self._spline_ids)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code if IGA operation is unsuccessful.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def warning_code(self) -> WarningCode:
        """Warning code if IGA operation is partially successful.
        """
        return self._warning_code

    @warning_code.setter
    def warning_code(self, value: WarningCode):
        self._warning_code = value

    @property
    def spline_ids(self) -> Iterable[int]:
        """Ids of the created spline.
        """
        return self._spline_ids

    @spline_ids.setter
    def spline_ids(self, value: Iterable[int]):
        self._spline_ids = value

class IGASpline(CoreObject):
    """Information of the spline.

    Parameters
    ----------
    model: Model
        Model to create a ``IGASpline`` object with default parameters.
    id: int, optional
        Unique id of the spline.
    json_data: dict, optional
        JSON dictionary to create a ``IGASpline`` object with provided parameters.

    Examples
    --------
    >>> iga_spline = prime.IGASpline(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            id: int):
        self._id = id

    def __init__(
            self,
            model: CommunicationManager=None,
            id: int = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``IGASpline`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``IGASpline`` object with default parameters.
        id: int, optional
            Unique id of the spline.
        json_data: dict, optional
            JSON dictionary to create a ``IGASpline`` object with provided parameters.

        Examples
        --------
        >>> iga_spline = prime.IGASpline(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["id"] if "id" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [id])
            if all_field_specified:
                self.__initialize(
                    id)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "IGASpline")
                    json_data = param_json["IGASpline"] if "IGASpline" in param_json else {}
                    self.__initialize(
                        id if id is not None else ( IGASpline._default_params["id"] if "id" in IGASpline._default_params else (json_data["id"] if "id" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            id: int = None):
        """Set the default values of the ``IGASpline`` object.

        Parameters
        ----------
        id: int, optional
            Unique id of the spline.
        """
        args = locals()
        [IGASpline._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``IGASpline`` object.

        Examples
        --------
        >>> IGASpline.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in IGASpline._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._id is not None:
            json_data["id"] = self._id
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "id :  %s" % (self._id)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def id(self) -> int:
        """Unique id of the spline.
        """
        return self._id

    @id.setter
    def id(self, value: int):
        self._id = value

class IGAUnstructuredSplineSurf(CoreObject):
    """Information of unstructured spline surface.

    Parameters
    ----------
    model: Model
        Model to create a ``IGAUnstructuredSplineSurf`` object with default parameters.
    id: int, optional
        Id of the unstructured spline surface.
    degree: int, optional
        Degree of the spline surface.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    spline_refinement_level: int, optional
        Refinement level for rendering of spline points.
    control_points: Iterable[float], optional
        Coordinates of the control points of the spline.
    spline_points: Iterable[float], optional
        Coordinates of the spline points.
    bad_spline_points_indices: Iterable[int], optional
        Node indices in the spline points list which has negative jacobian value.
    deviation_array: Iterable[float], optional
        Deviation value from the spline point to the model geometry.
    invalid_jacobian_elements_count: int, optional
        Count of elements with negative jacobian.
    average_mesh_size: float, optional
        Reference length to compute deviation.
    elements_count: int, optional
        Count of shell elements.
    shell_thickness: float, optional
        Thickness of shell.
    json_data: dict, optional
        JSON dictionary to create a ``IGAUnstructuredSplineSurf`` object with provided parameters.

    Examples
    --------
    >>> iga_unstructured_spline_surf = prime.IGAUnstructuredSplineSurf(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            id: int,
            degree: int,
            spline_refinement_level: int,
            control_points: Iterable[float],
            spline_points: Iterable[float],
            bad_spline_points_indices: Iterable[int],
            deviation_array: Iterable[float],
            invalid_jacobian_elements_count: int,
            average_mesh_size: float,
            elements_count: int,
            shell_thickness: float):
        self._id = id
        self._degree = degree
        self._spline_refinement_level = spline_refinement_level
        self._control_points = control_points if isinstance(control_points, np.ndarray) else np.array(control_points, dtype=np.double) if control_points is not None else None
        self._spline_points = spline_points if isinstance(spline_points, np.ndarray) else np.array(spline_points, dtype=np.double) if spline_points is not None else None
        self._bad_spline_points_indices = bad_spline_points_indices if isinstance(bad_spline_points_indices, np.ndarray) else np.array(bad_spline_points_indices, dtype=np.int32) if bad_spline_points_indices is not None else None
        self._deviation_array = deviation_array if isinstance(deviation_array, np.ndarray) else np.array(deviation_array, dtype=np.double) if deviation_array is not None else None
        self._invalid_jacobian_elements_count = invalid_jacobian_elements_count
        self._average_mesh_size = average_mesh_size
        self._elements_count = elements_count
        self._shell_thickness = shell_thickness

    def __init__(
            self,
            model: CommunicationManager=None,
            id: int = None,
            degree: int = None,
            spline_refinement_level: int = None,
            control_points: Iterable[float] = None,
            spline_points: Iterable[float] = None,
            bad_spline_points_indices: Iterable[int] = None,
            deviation_array: Iterable[float] = None,
            invalid_jacobian_elements_count: int = None,
            average_mesh_size: float = None,
            elements_count: int = None,
            shell_thickness: float = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``IGAUnstructuredSplineSurf`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``IGAUnstructuredSplineSurf`` object with default parameters.
        id: int, optional
            Id of the unstructured spline surface.
        degree: int, optional
            Degree of the spline surface.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        spline_refinement_level: int, optional
            Refinement level for rendering of spline points.
        control_points: Iterable[float], optional
            Coordinates of the control points of the spline.
        spline_points: Iterable[float], optional
            Coordinates of the spline points.
        bad_spline_points_indices: Iterable[int], optional
            Node indices in the spline points list which has negative jacobian value.
        deviation_array: Iterable[float], optional
            Deviation value from the spline point to the model geometry.
        invalid_jacobian_elements_count: int, optional
            Count of elements with negative jacobian.
        average_mesh_size: float, optional
            Reference length to compute deviation.
        elements_count: int, optional
            Count of shell elements.
        shell_thickness: float, optional
            Thickness of shell.
        json_data: dict, optional
            JSON dictionary to create a ``IGAUnstructuredSplineSurf`` object with provided parameters.

        Examples
        --------
        >>> iga_unstructured_spline_surf = prime.IGAUnstructuredSplineSurf(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["id"] if "id" in json_data else None,
                json_data["degree"] if "degree" in json_data else None,
                json_data["splineRefinementLevel"] if "splineRefinementLevel" in json_data else None,
                json_data["controlPoints"] if "controlPoints" in json_data else None,
                json_data["splinePoints"] if "splinePoints" in json_data else None,
                json_data["badSplinePointsIndices"] if "badSplinePointsIndices" in json_data else None,
                json_data["deviationArray"] if "deviationArray" in json_data else None,
                json_data["invalidJacobianElementsCount"] if "invalidJacobianElementsCount" in json_data else None,
                json_data["averageMeshSize"] if "averageMeshSize" in json_data else None,
                json_data["elementsCount"] if "elementsCount" in json_data else None,
                json_data["shellThickness"] if "shellThickness" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [id, degree, spline_refinement_level, control_points, spline_points, bad_spline_points_indices, deviation_array, invalid_jacobian_elements_count, average_mesh_size, elements_count, shell_thickness])
            if all_field_specified:
                self.__initialize(
                    id,
                    degree,
                    spline_refinement_level,
                    control_points,
                    spline_points,
                    bad_spline_points_indices,
                    deviation_array,
                    invalid_jacobian_elements_count,
                    average_mesh_size,
                    elements_count,
                    shell_thickness)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "IGAUnstructuredSplineSurf")
                    json_data = param_json["IGAUnstructuredSplineSurf"] if "IGAUnstructuredSplineSurf" in param_json else {}
                    self.__initialize(
                        id if id is not None else ( IGAUnstructuredSplineSurf._default_params["id"] if "id" in IGAUnstructuredSplineSurf._default_params else (json_data["id"] if "id" in json_data else None)),
                        degree if degree is not None else ( IGAUnstructuredSplineSurf._default_params["degree"] if "degree" in IGAUnstructuredSplineSurf._default_params else (json_data["degree"] if "degree" in json_data else None)),
                        spline_refinement_level if spline_refinement_level is not None else ( IGAUnstructuredSplineSurf._default_params["spline_refinement_level"] if "spline_refinement_level" in IGAUnstructuredSplineSurf._default_params else (json_data["splineRefinementLevel"] if "splineRefinementLevel" in json_data else None)),
                        control_points if control_points is not None else ( IGAUnstructuredSplineSurf._default_params["control_points"] if "control_points" in IGAUnstructuredSplineSurf._default_params else (json_data["controlPoints"] if "controlPoints" in json_data else None)),
                        spline_points if spline_points is not None else ( IGAUnstructuredSplineSurf._default_params["spline_points"] if "spline_points" in IGAUnstructuredSplineSurf._default_params else (json_data["splinePoints"] if "splinePoints" in json_data else None)),
                        bad_spline_points_indices if bad_spline_points_indices is not None else ( IGAUnstructuredSplineSurf._default_params["bad_spline_points_indices"] if "bad_spline_points_indices" in IGAUnstructuredSplineSurf._default_params else (json_data["badSplinePointsIndices"] if "badSplinePointsIndices" in json_data else None)),
                        deviation_array if deviation_array is not None else ( IGAUnstructuredSplineSurf._default_params["deviation_array"] if "deviation_array" in IGAUnstructuredSplineSurf._default_params else (json_data["deviationArray"] if "deviationArray" in json_data else None)),
                        invalid_jacobian_elements_count if invalid_jacobian_elements_count is not None else ( IGAUnstructuredSplineSurf._default_params["invalid_jacobian_elements_count"] if "invalid_jacobian_elements_count" in IGAUnstructuredSplineSurf._default_params else (json_data["invalidJacobianElementsCount"] if "invalidJacobianElementsCount" in json_data else None)),
                        average_mesh_size if average_mesh_size is not None else ( IGAUnstructuredSplineSurf._default_params["average_mesh_size"] if "average_mesh_size" in IGAUnstructuredSplineSurf._default_params else (json_data["averageMeshSize"] if "averageMeshSize" in json_data else None)),
                        elements_count if elements_count is not None else ( IGAUnstructuredSplineSurf._default_params["elements_count"] if "elements_count" in IGAUnstructuredSplineSurf._default_params else (json_data["elementsCount"] if "elementsCount" in json_data else None)),
                        shell_thickness if shell_thickness is not None else ( IGAUnstructuredSplineSurf._default_params["shell_thickness"] if "shell_thickness" in IGAUnstructuredSplineSurf._default_params else (json_data["shellThickness"] if "shellThickness" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            id: int = None,
            degree: int = None,
            spline_refinement_level: int = None,
            control_points: Iterable[float] = None,
            spline_points: Iterable[float] = None,
            bad_spline_points_indices: Iterable[int] = None,
            deviation_array: Iterable[float] = None,
            invalid_jacobian_elements_count: int = None,
            average_mesh_size: float = None,
            elements_count: int = None,
            shell_thickness: float = None):
        """Set the default values of the ``IGAUnstructuredSplineSurf`` object.

        Parameters
        ----------
        id: int, optional
            Id of the unstructured spline surface.
        degree: int, optional
            Degree of the spline surface.
        spline_refinement_level: int, optional
            Refinement level for rendering of spline points.
        control_points: Iterable[float], optional
            Coordinates of the control points of the spline.
        spline_points: Iterable[float], optional
            Coordinates of the spline points.
        bad_spline_points_indices: Iterable[int], optional
            Node indices in the spline points list which has negative jacobian value.
        deviation_array: Iterable[float], optional
            Deviation value from the spline point to the model geometry.
        invalid_jacobian_elements_count: int, optional
            Count of elements with negative jacobian.
        average_mesh_size: float, optional
            Reference length to compute deviation.
        elements_count: int, optional
            Count of shell elements.
        shell_thickness: float, optional
            Thickness of shell.
        """
        args = locals()
        [IGAUnstructuredSplineSurf._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``IGAUnstructuredSplineSurf`` object.

        Examples
        --------
        >>> IGAUnstructuredSplineSurf.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in IGAUnstructuredSplineSurf._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._id is not None:
            json_data["id"] = self._id
        if self._degree is not None:
            json_data["degree"] = self._degree
        if self._spline_refinement_level is not None:
            json_data["splineRefinementLevel"] = self._spline_refinement_level
        if self._control_points is not None:
            json_data["controlPoints"] = self._control_points
        if self._spline_points is not None:
            json_data["splinePoints"] = self._spline_points
        if self._bad_spline_points_indices is not None:
            json_data["badSplinePointsIndices"] = self._bad_spline_points_indices
        if self._deviation_array is not None:
            json_data["deviationArray"] = self._deviation_array
        if self._invalid_jacobian_elements_count is not None:
            json_data["invalidJacobianElementsCount"] = self._invalid_jacobian_elements_count
        if self._average_mesh_size is not None:
            json_data["averageMeshSize"] = self._average_mesh_size
        if self._elements_count is not None:
            json_data["elementsCount"] = self._elements_count
        if self._shell_thickness is not None:
            json_data["shellThickness"] = self._shell_thickness
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "id :  %s\ndegree :  %s\nspline_refinement_level :  %s\ncontrol_points :  %s\nspline_points :  %s\nbad_spline_points_indices :  %s\ndeviation_array :  %s\ninvalid_jacobian_elements_count :  %s\naverage_mesh_size :  %s\nelements_count :  %s\nshell_thickness :  %s" % (self._id, self._degree, self._spline_refinement_level, self._control_points, self._spline_points, self._bad_spline_points_indices, self._deviation_array, self._invalid_jacobian_elements_count, self._average_mesh_size, self._elements_count, self._shell_thickness)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def id(self) -> int:
        """Id of the unstructured spline surface.
        """
        return self._id

    @id.setter
    def id(self, value: int):
        self._id = value

    @property
    def degree(self) -> int:
        """Degree of the spline surface.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._degree

    @degree.setter
    def degree(self, value: int):
        self._degree = value

    @property
    def spline_refinement_level(self) -> int:
        """Refinement level for rendering of spline points.
        """
        return self._spline_refinement_level

    @spline_refinement_level.setter
    def spline_refinement_level(self, value: int):
        self._spline_refinement_level = value

    @property
    def control_points(self) -> Iterable[float]:
        """Coordinates of the control points of the spline.
        """
        return self._control_points

    @control_points.setter
    def control_points(self, value: Iterable[float]):
        self._control_points = value

    @property
    def spline_points(self) -> Iterable[float]:
        """Coordinates of the spline points.
        """
        return self._spline_points

    @spline_points.setter
    def spline_points(self, value: Iterable[float]):
        self._spline_points = value

    @property
    def bad_spline_points_indices(self) -> Iterable[int]:
        """Node indices in the spline points list which has negative jacobian value.
        """
        return self._bad_spline_points_indices

    @bad_spline_points_indices.setter
    def bad_spline_points_indices(self, value: Iterable[int]):
        self._bad_spline_points_indices = value

    @property
    def deviation_array(self) -> Iterable[float]:
        """Deviation value from the spline point to the model geometry.
        """
        return self._deviation_array

    @deviation_array.setter
    def deviation_array(self, value: Iterable[float]):
        self._deviation_array = value

    @property
    def invalid_jacobian_elements_count(self) -> int:
        """Count of elements with negative jacobian.
        """
        return self._invalid_jacobian_elements_count

    @invalid_jacobian_elements_count.setter
    def invalid_jacobian_elements_count(self, value: int):
        self._invalid_jacobian_elements_count = value

    @property
    def average_mesh_size(self) -> float:
        """Reference length to compute deviation.
        """
        return self._average_mesh_size

    @average_mesh_size.setter
    def average_mesh_size(self, value: float):
        self._average_mesh_size = value

    @property
    def elements_count(self) -> int:
        """Count of shell elements.
        """
        return self._elements_count

    @elements_count.setter
    def elements_count(self, value: int):
        self._elements_count = value

    @property
    def shell_thickness(self) -> float:
        """Thickness of shell.
        """
        return self._shell_thickness

    @shell_thickness.setter
    def shell_thickness(self, value: float):
        self._shell_thickness = value

class IGAUnstructuredSplineSolid(CoreObject):
    """Information of unstructured spline solid.

    Parameters
    ----------
    model: Model
        Model to create a ``IGAUnstructuredSplineSolid`` object with default parameters.
    id: int, optional
        Id of the unstructured spline solid.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    degree: int, optional
        Degree of the spline solid.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    invalid_jacobian_elements_count: int, optional
        Count of elements with negative jacobian.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``IGAUnstructuredSplineSolid`` object with provided parameters.

    Examples
    --------
    >>> iga_unstructured_spline_solid = prime.IGAUnstructuredSplineSolid(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            id: int,
            degree: int,
            invalid_jacobian_elements_count: int):
        self._id = id
        self._degree = degree
        self._invalid_jacobian_elements_count = invalid_jacobian_elements_count

    def __init__(
            self,
            model: CommunicationManager=None,
            id: int = None,
            degree: int = None,
            invalid_jacobian_elements_count: int = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``IGAUnstructuredSplineSolid`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``IGAUnstructuredSplineSolid`` object with default parameters.
        id: int, optional
            Id of the unstructured spline solid.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        degree: int, optional
            Degree of the spline solid.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        invalid_jacobian_elements_count: int, optional
            Count of elements with negative jacobian.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``IGAUnstructuredSplineSolid`` object with provided parameters.

        Examples
        --------
        >>> iga_unstructured_spline_solid = prime.IGAUnstructuredSplineSolid(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["id"] if "id" in json_data else None,
                json_data["degree"] if "degree" in json_data else None,
                json_data["invalidJacobianElementsCount"] if "invalidJacobianElementsCount" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [id, degree, invalid_jacobian_elements_count])
            if all_field_specified:
                self.__initialize(
                    id,
                    degree,
                    invalid_jacobian_elements_count)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "IGAUnstructuredSplineSolid")
                    json_data = param_json["IGAUnstructuredSplineSolid"] if "IGAUnstructuredSplineSolid" in param_json else {}
                    self.__initialize(
                        id if id is not None else ( IGAUnstructuredSplineSolid._default_params["id"] if "id" in IGAUnstructuredSplineSolid._default_params else (json_data["id"] if "id" in json_data else None)),
                        degree if degree is not None else ( IGAUnstructuredSplineSolid._default_params["degree"] if "degree" in IGAUnstructuredSplineSolid._default_params else (json_data["degree"] if "degree" in json_data else None)),
                        invalid_jacobian_elements_count if invalid_jacobian_elements_count is not None else ( IGAUnstructuredSplineSolid._default_params["invalid_jacobian_elements_count"] if "invalid_jacobian_elements_count" in IGAUnstructuredSplineSolid._default_params else (json_data["invalidJacobianElementsCount"] if "invalidJacobianElementsCount" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            id: int = None,
            degree: int = None,
            invalid_jacobian_elements_count: int = None):
        """Set the default values of the ``IGAUnstructuredSplineSolid`` object.

        Parameters
        ----------
        id: int, optional
            Id of the unstructured spline solid.
        degree: int, optional
            Degree of the spline solid.
        invalid_jacobian_elements_count: int, optional
            Count of elements with negative jacobian.
        """
        args = locals()
        [IGAUnstructuredSplineSolid._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``IGAUnstructuredSplineSolid`` object.

        Examples
        --------
        >>> IGAUnstructuredSplineSolid.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in IGAUnstructuredSplineSolid._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._id is not None:
            json_data["id"] = self._id
        if self._degree is not None:
            json_data["degree"] = self._degree
        if self._invalid_jacobian_elements_count is not None:
            json_data["invalidJacobianElementsCount"] = self._invalid_jacobian_elements_count
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "id :  %s\ndegree :  %s\ninvalid_jacobian_elements_count :  %s" % (self._id, self._degree, self._invalid_jacobian_elements_count)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def id(self) -> int:
        """Id of the unstructured spline solid.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._id

    @id.setter
    def id(self, value: int):
        self._id = value

    @property
    def degree(self) -> int:
        """Degree of the spline solid.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._degree

    @degree.setter
    def degree(self, value: int):
        self._degree = value

    @property
    def invalid_jacobian_elements_count(self) -> int:
        """Count of elements with negative jacobian.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._invalid_jacobian_elements_count

    @invalid_jacobian_elements_count.setter
    def invalid_jacobian_elements_count(self, value: int):
        self._invalid_jacobian_elements_count = value

class BoundaryFittedSplineParams(CoreObject):
    """Boundary fitted spline fitting parameters.

    Parameters
    ----------
    model: Model
        Model to create a ``BoundaryFittedSplineParams`` object with default parameters.
    degree_u: int, optional
        Degree of spline in u direction.
    degree_v: int, optional
        Degree of spline in v direction.
    degree_w: int, optional
        Degree of spline in w direction.
    refinement_fraction_u: float, optional
        Fraction of input mesh size that sets the control points size in u direction. This is used in program controlled control points selection mode.
    refinement_fraction_v: float, optional
        Fraction of input mesh size that sets the control points size in v direction. This is used in program controlled control points selection mode.
    refinement_fraction_w: float, optional
        Fraction of input mesh size that sets the control points size in w direction. This is used in program controlled control points selection mode.
    control_points_count_u: int, optional
        Spline control points count in U direction. Used in manual control points selection mode.
    control_points_count_v: int, optional
        Spline control points count in V direction. Used in manual control points selection mode.
    control_points_count_w: int, optional
        Spline control points count in W direction. Used in manual control points selection mode.
    n_refine: int, optional
        Spline refinement level for rendering.
    control_point_selection_type: ControlPointSelection, optional
        Spline control points selection type.
    json_data: dict, optional
        JSON dictionary to create a ``BoundaryFittedSplineParams`` object with provided parameters.

    Examples
    --------
    >>> boundary_fitted_spline_params = prime.BoundaryFittedSplineParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            degree_u: int,
            degree_v: int,
            degree_w: int,
            refinement_fraction_u: float,
            refinement_fraction_v: float,
            refinement_fraction_w: float,
            control_points_count_u: int,
            control_points_count_v: int,
            control_points_count_w: int,
            n_refine: int,
            control_point_selection_type: ControlPointSelection):
        self._degree_u = degree_u
        self._degree_v = degree_v
        self._degree_w = degree_w
        self._refinement_fraction_u = refinement_fraction_u
        self._refinement_fraction_v = refinement_fraction_v
        self._refinement_fraction_w = refinement_fraction_w
        self._control_points_count_u = control_points_count_u
        self._control_points_count_v = control_points_count_v
        self._control_points_count_w = control_points_count_w
        self._n_refine = n_refine
        self._control_point_selection_type = ControlPointSelection(control_point_selection_type)

    def __init__(
            self,
            model: CommunicationManager=None,
            degree_u: int = None,
            degree_v: int = None,
            degree_w: int = None,
            refinement_fraction_u: float = None,
            refinement_fraction_v: float = None,
            refinement_fraction_w: float = None,
            control_points_count_u: int = None,
            control_points_count_v: int = None,
            control_points_count_w: int = None,
            n_refine: int = None,
            control_point_selection_type: ControlPointSelection = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``BoundaryFittedSplineParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``BoundaryFittedSplineParams`` object with default parameters.
        degree_u: int, optional
            Degree of spline in u direction.
        degree_v: int, optional
            Degree of spline in v direction.
        degree_w: int, optional
            Degree of spline in w direction.
        refinement_fraction_u: float, optional
            Fraction of input mesh size that sets the control points size in u direction. This is used in program controlled control points selection mode.
        refinement_fraction_v: float, optional
            Fraction of input mesh size that sets the control points size in v direction. This is used in program controlled control points selection mode.
        refinement_fraction_w: float, optional
            Fraction of input mesh size that sets the control points size in w direction. This is used in program controlled control points selection mode.
        control_points_count_u: int, optional
            Spline control points count in U direction. Used in manual control points selection mode.
        control_points_count_v: int, optional
            Spline control points count in V direction. Used in manual control points selection mode.
        control_points_count_w: int, optional
            Spline control points count in W direction. Used in manual control points selection mode.
        n_refine: int, optional
            Spline refinement level for rendering.
        control_point_selection_type: ControlPointSelection, optional
            Spline control points selection type.
        json_data: dict, optional
            JSON dictionary to create a ``BoundaryFittedSplineParams`` object with provided parameters.

        Examples
        --------
        >>> boundary_fitted_spline_params = prime.BoundaryFittedSplineParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["degreeU"] if "degreeU" in json_data else None,
                json_data["degreeV"] if "degreeV" in json_data else None,
                json_data["degreeW"] if "degreeW" in json_data else None,
                json_data["refinementFractionU"] if "refinementFractionU" in json_data else None,
                json_data["refinementFractionV"] if "refinementFractionV" in json_data else None,
                json_data["refinementFractionW"] if "refinementFractionW" in json_data else None,
                json_data["controlPointsCountU"] if "controlPointsCountU" in json_data else None,
                json_data["controlPointsCountV"] if "controlPointsCountV" in json_data else None,
                json_data["controlPointsCountW"] if "controlPointsCountW" in json_data else None,
                json_data["nRefine"] if "nRefine" in json_data else None,
                ControlPointSelection(json_data["controlPointSelectionType"] if "controlPointSelectionType" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [degree_u, degree_v, degree_w, refinement_fraction_u, refinement_fraction_v, refinement_fraction_w, control_points_count_u, control_points_count_v, control_points_count_w, n_refine, control_point_selection_type])
            if all_field_specified:
                self.__initialize(
                    degree_u,
                    degree_v,
                    degree_w,
                    refinement_fraction_u,
                    refinement_fraction_v,
                    refinement_fraction_w,
                    control_points_count_u,
                    control_points_count_v,
                    control_points_count_w,
                    n_refine,
                    control_point_selection_type)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "BoundaryFittedSplineParams")
                    json_data = param_json["BoundaryFittedSplineParams"] if "BoundaryFittedSplineParams" in param_json else {}
                    self.__initialize(
                        degree_u if degree_u is not None else ( BoundaryFittedSplineParams._default_params["degree_u"] if "degree_u" in BoundaryFittedSplineParams._default_params else (json_data["degreeU"] if "degreeU" in json_data else None)),
                        degree_v if degree_v is not None else ( BoundaryFittedSplineParams._default_params["degree_v"] if "degree_v" in BoundaryFittedSplineParams._default_params else (json_data["degreeV"] if "degreeV" in json_data else None)),
                        degree_w if degree_w is not None else ( BoundaryFittedSplineParams._default_params["degree_w"] if "degree_w" in BoundaryFittedSplineParams._default_params else (json_data["degreeW"] if "degreeW" in json_data else None)),
                        refinement_fraction_u if refinement_fraction_u is not None else ( BoundaryFittedSplineParams._default_params["refinement_fraction_u"] if "refinement_fraction_u" in BoundaryFittedSplineParams._default_params else (json_data["refinementFractionU"] if "refinementFractionU" in json_data else None)),
                        refinement_fraction_v if refinement_fraction_v is not None else ( BoundaryFittedSplineParams._default_params["refinement_fraction_v"] if "refinement_fraction_v" in BoundaryFittedSplineParams._default_params else (json_data["refinementFractionV"] if "refinementFractionV" in json_data else None)),
                        refinement_fraction_w if refinement_fraction_w is not None else ( BoundaryFittedSplineParams._default_params["refinement_fraction_w"] if "refinement_fraction_w" in BoundaryFittedSplineParams._default_params else (json_data["refinementFractionW"] if "refinementFractionW" in json_data else None)),
                        control_points_count_u if control_points_count_u is not None else ( BoundaryFittedSplineParams._default_params["control_points_count_u"] if "control_points_count_u" in BoundaryFittedSplineParams._default_params else (json_data["controlPointsCountU"] if "controlPointsCountU" in json_data else None)),
                        control_points_count_v if control_points_count_v is not None else ( BoundaryFittedSplineParams._default_params["control_points_count_v"] if "control_points_count_v" in BoundaryFittedSplineParams._default_params else (json_data["controlPointsCountV"] if "controlPointsCountV" in json_data else None)),
                        control_points_count_w if control_points_count_w is not None else ( BoundaryFittedSplineParams._default_params["control_points_count_w"] if "control_points_count_w" in BoundaryFittedSplineParams._default_params else (json_data["controlPointsCountW"] if "controlPointsCountW" in json_data else None)),
                        n_refine if n_refine is not None else ( BoundaryFittedSplineParams._default_params["n_refine"] if "n_refine" in BoundaryFittedSplineParams._default_params else (json_data["nRefine"] if "nRefine" in json_data else None)),
                        control_point_selection_type if control_point_selection_type is not None else ( BoundaryFittedSplineParams._default_params["control_point_selection_type"] if "control_point_selection_type" in BoundaryFittedSplineParams._default_params else ControlPointSelection(json_data["controlPointSelectionType"] if "controlPointSelectionType" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            degree_u: int = None,
            degree_v: int = None,
            degree_w: int = None,
            refinement_fraction_u: float = None,
            refinement_fraction_v: float = None,
            refinement_fraction_w: float = None,
            control_points_count_u: int = None,
            control_points_count_v: int = None,
            control_points_count_w: int = None,
            n_refine: int = None,
            control_point_selection_type: ControlPointSelection = None):
        """Set the default values of the ``BoundaryFittedSplineParams`` object.

        Parameters
        ----------
        degree_u: int, optional
            Degree of spline in u direction.
        degree_v: int, optional
            Degree of spline in v direction.
        degree_w: int, optional
            Degree of spline in w direction.
        refinement_fraction_u: float, optional
            Fraction of input mesh size that sets the control points size in u direction. This is used in program controlled control points selection mode.
        refinement_fraction_v: float, optional
            Fraction of input mesh size that sets the control points size in v direction. This is used in program controlled control points selection mode.
        refinement_fraction_w: float, optional
            Fraction of input mesh size that sets the control points size in w direction. This is used in program controlled control points selection mode.
        control_points_count_u: int, optional
            Spline control points count in U direction. Used in manual control points selection mode.
        control_points_count_v: int, optional
            Spline control points count in V direction. Used in manual control points selection mode.
        control_points_count_w: int, optional
            Spline control points count in W direction. Used in manual control points selection mode.
        n_refine: int, optional
            Spline refinement level for rendering.
        control_point_selection_type: ControlPointSelection, optional
            Spline control points selection type.
        """
        args = locals()
        [BoundaryFittedSplineParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``BoundaryFittedSplineParams`` object.

        Examples
        --------
        >>> BoundaryFittedSplineParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in BoundaryFittedSplineParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._degree_u is not None:
            json_data["degreeU"] = self._degree_u
        if self._degree_v is not None:
            json_data["degreeV"] = self._degree_v
        if self._degree_w is not None:
            json_data["degreeW"] = self._degree_w
        if self._refinement_fraction_u is not None:
            json_data["refinementFractionU"] = self._refinement_fraction_u
        if self._refinement_fraction_v is not None:
            json_data["refinementFractionV"] = self._refinement_fraction_v
        if self._refinement_fraction_w is not None:
            json_data["refinementFractionW"] = self._refinement_fraction_w
        if self._control_points_count_u is not None:
            json_data["controlPointsCountU"] = self._control_points_count_u
        if self._control_points_count_v is not None:
            json_data["controlPointsCountV"] = self._control_points_count_v
        if self._control_points_count_w is not None:
            json_data["controlPointsCountW"] = self._control_points_count_w
        if self._n_refine is not None:
            json_data["nRefine"] = self._n_refine
        if self._control_point_selection_type is not None:
            json_data["controlPointSelectionType"] = self._control_point_selection_type
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "degree_u :  %s\ndegree_v :  %s\ndegree_w :  %s\nrefinement_fraction_u :  %s\nrefinement_fraction_v :  %s\nrefinement_fraction_w :  %s\ncontrol_points_count_u :  %s\ncontrol_points_count_v :  %s\ncontrol_points_count_w :  %s\nn_refine :  %s\ncontrol_point_selection_type :  %s" % (self._degree_u, self._degree_v, self._degree_w, self._refinement_fraction_u, self._refinement_fraction_v, self._refinement_fraction_w, self._control_points_count_u, self._control_points_count_v, self._control_points_count_w, self._n_refine, self._control_point_selection_type)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def degree_u(self) -> int:
        """Degree of spline in u direction.
        """
        return self._degree_u

    @degree_u.setter
    def degree_u(self, value: int):
        self._degree_u = value

    @property
    def degree_v(self) -> int:
        """Degree of spline in v direction.
        """
        return self._degree_v

    @degree_v.setter
    def degree_v(self, value: int):
        self._degree_v = value

    @property
    def degree_w(self) -> int:
        """Degree of spline in w direction.
        """
        return self._degree_w

    @degree_w.setter
    def degree_w(self, value: int):
        self._degree_w = value

    @property
    def refinement_fraction_u(self) -> float:
        """Fraction of input mesh size that sets the control points size in u direction. This is used in program controlled control points selection mode.
        """
        return self._refinement_fraction_u

    @refinement_fraction_u.setter
    def refinement_fraction_u(self, value: float):
        self._refinement_fraction_u = value

    @property
    def refinement_fraction_v(self) -> float:
        """Fraction of input mesh size that sets the control points size in v direction. This is used in program controlled control points selection mode.
        """
        return self._refinement_fraction_v

    @refinement_fraction_v.setter
    def refinement_fraction_v(self, value: float):
        self._refinement_fraction_v = value

    @property
    def refinement_fraction_w(self) -> float:
        """Fraction of input mesh size that sets the control points size in w direction. This is used in program controlled control points selection mode.
        """
        return self._refinement_fraction_w

    @refinement_fraction_w.setter
    def refinement_fraction_w(self, value: float):
        self._refinement_fraction_w = value

    @property
    def control_points_count_u(self) -> int:
        """Spline control points count in U direction. Used in manual control points selection mode.
        """
        return self._control_points_count_u

    @control_points_count_u.setter
    def control_points_count_u(self, value: int):
        self._control_points_count_u = value

    @property
    def control_points_count_v(self) -> int:
        """Spline control points count in V direction. Used in manual control points selection mode.
        """
        return self._control_points_count_v

    @control_points_count_v.setter
    def control_points_count_v(self, value: int):
        self._control_points_count_v = value

    @property
    def control_points_count_w(self) -> int:
        """Spline control points count in W direction. Used in manual control points selection mode.
        """
        return self._control_points_count_w

    @control_points_count_w.setter
    def control_points_count_w(self, value: int):
        self._control_points_count_w = value

    @property
    def n_refine(self) -> int:
        """Spline refinement level for rendering.
        """
        return self._n_refine

    @n_refine.setter
    def n_refine(self, value: int):
        self._n_refine = value

    @property
    def control_point_selection_type(self) -> ControlPointSelection:
        """Spline control points selection type.
        """
        return self._control_point_selection_type

    @control_point_selection_type.setter
    def control_point_selection_type(self, value: ControlPointSelection):
        self._control_point_selection_type = value

class QuadToSplineParams(CoreObject):
    """Parameters to control conversion of quadrilateral mesh to spline.

    Parameters
    ----------
    model: Model
        Model to create a ``QuadToSplineParams`` object with default parameters.
    feature_capture_type: SplineFeatureCaptureType, optional
        Feature capture options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    feature_angle: float, optional
        Angle to capture the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    corner_angle: float, optional
        Corner angle of the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    shell_thickness: float, optional
        Thickness of shell.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    solid_shell: bool, optional
        Solid shell option. Set true to generate solid shell spline, and set false to generate surface spline.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    separate_by_zone: bool, optional
        Option to separate IGA shell regions by zone. If set to true, it creates LS-Dyna part per zone while exporting IGA .k file and if set to false, it creates a single LS-Dyna part per Prime part, irrespective of the zones.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    zone_name_shell_thickness_pairs: Dict[str, Union[str, int, float, bool]], optional
        Zone name and thickness pair list. For example, {"Zone1Name": Zone1Thickness, "Zone2Name": Zone2Thickness, ...}.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    continuity: SplineContinuityType, optional
        Spline Continuity options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    project_on_geometry: bool, optional
        Option to project on geometry.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    use_projection_scope: bool, optional
        Option to use projection scope.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    projection_scope: ScopeDefinition, optional
        Scope to evaluate entities for projection.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``QuadToSplineParams`` object with provided parameters.

    Examples
    --------
    >>> quad_to_spline_params = prime.QuadToSplineParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            feature_capture_type: SplineFeatureCaptureType,
            feature_angle: float,
            corner_angle: float,
            shell_thickness: float,
            solid_shell: bool,
            separate_by_zone: bool,
            zone_name_shell_thickness_pairs: Dict[str, Union[str, int, float, bool]],
            continuity: SplineContinuityType,
            project_on_geometry: bool,
            use_projection_scope: bool,
            projection_scope: ScopeDefinition):
        self._feature_capture_type = SplineFeatureCaptureType(feature_capture_type)
        self._feature_angle = feature_angle
        self._corner_angle = corner_angle
        self._shell_thickness = shell_thickness
        self._solid_shell = solid_shell
        self._separate_by_zone = separate_by_zone
        self._zone_name_shell_thickness_pairs = zone_name_shell_thickness_pairs
        self._continuity = SplineContinuityType(continuity)
        self._project_on_geometry = project_on_geometry
        self._use_projection_scope = use_projection_scope
        self._projection_scope = projection_scope

    def __init__(
            self,
            model: CommunicationManager=None,
            feature_capture_type: SplineFeatureCaptureType = None,
            feature_angle: float = None,
            corner_angle: float = None,
            shell_thickness: float = None,
            solid_shell: bool = None,
            separate_by_zone: bool = None,
            zone_name_shell_thickness_pairs: Dict[str, Union[str, int, float, bool]] = None,
            continuity: SplineContinuityType = None,
            project_on_geometry: bool = None,
            use_projection_scope: bool = None,
            projection_scope: ScopeDefinition = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``QuadToSplineParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``QuadToSplineParams`` object with default parameters.
        feature_capture_type: SplineFeatureCaptureType, optional
            Feature capture options.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        feature_angle: float, optional
            Angle to capture the feature.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        corner_angle: float, optional
            Corner angle of the feature.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        shell_thickness: float, optional
            Thickness of shell.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        solid_shell: bool, optional
            Solid shell option. Set true to generate solid shell spline, and set false to generate surface spline.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        separate_by_zone: bool, optional
            Option to separate IGA shell regions by zone. If set to true, it creates LS-Dyna part per zone while exporting IGA .k file and if set to false, it creates a single LS-Dyna part per Prime part, irrespective of the zones.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        zone_name_shell_thickness_pairs: Dict[str, Union[str, int, float, bool]], optional
            Zone name and thickness pair list. For example, {"Zone1Name": Zone1Thickness, "Zone2Name": Zone2Thickness, ...}.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        continuity: SplineContinuityType, optional
            Spline Continuity options.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        project_on_geometry: bool, optional
            Option to project on geometry.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        use_projection_scope: bool, optional
            Option to use projection scope.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        projection_scope: ScopeDefinition, optional
            Scope to evaluate entities for projection.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``QuadToSplineParams`` object with provided parameters.

        Examples
        --------
        >>> quad_to_spline_params = prime.QuadToSplineParams(model = model)
        """
        if json_data:
            self.__initialize(
                SplineFeatureCaptureType(json_data["featureCaptureType"] if "featureCaptureType" in json_data else None),
                json_data["featureAngle"] if "featureAngle" in json_data else None,
                json_data["cornerAngle"] if "cornerAngle" in json_data else None,
                json_data["shellThickness"] if "shellThickness" in json_data else None,
                json_data["solidShell"] if "solidShell" in json_data else None,
                json_data["separateByZone"] if "separateByZone" in json_data else None,
                json_data["zoneNameShellThicknessPairs"] if "zoneNameShellThicknessPairs" in json_data else None,
                SplineContinuityType(json_data["continuity"] if "continuity" in json_data else None),
                json_data["projectOnGeometry"] if "projectOnGeometry" in json_data else None,
                json_data["useProjectionScope"] if "useProjectionScope" in json_data else None,
                ScopeDefinition(model = model, json_data = json_data["projectionScope"] if "projectionScope" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [feature_capture_type, feature_angle, corner_angle, shell_thickness, solid_shell, separate_by_zone, zone_name_shell_thickness_pairs, continuity, project_on_geometry, use_projection_scope, projection_scope])
            if all_field_specified:
                self.__initialize(
                    feature_capture_type,
                    feature_angle,
                    corner_angle,
                    shell_thickness,
                    solid_shell,
                    separate_by_zone,
                    zone_name_shell_thickness_pairs,
                    continuity,
                    project_on_geometry,
                    use_projection_scope,
                    projection_scope)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "QuadToSplineParams")
                    json_data = param_json["QuadToSplineParams"] if "QuadToSplineParams" in param_json else {}
                    self.__initialize(
                        feature_capture_type if feature_capture_type is not None else ( QuadToSplineParams._default_params["feature_capture_type"] if "feature_capture_type" in QuadToSplineParams._default_params else SplineFeatureCaptureType(json_data["featureCaptureType"] if "featureCaptureType" in json_data else None)),
                        feature_angle if feature_angle is not None else ( QuadToSplineParams._default_params["feature_angle"] if "feature_angle" in QuadToSplineParams._default_params else (json_data["featureAngle"] if "featureAngle" in json_data else None)),
                        corner_angle if corner_angle is not None else ( QuadToSplineParams._default_params["corner_angle"] if "corner_angle" in QuadToSplineParams._default_params else (json_data["cornerAngle"] if "cornerAngle" in json_data else None)),
                        shell_thickness if shell_thickness is not None else ( QuadToSplineParams._default_params["shell_thickness"] if "shell_thickness" in QuadToSplineParams._default_params else (json_data["shellThickness"] if "shellThickness" in json_data else None)),
                        solid_shell if solid_shell is not None else ( QuadToSplineParams._default_params["solid_shell"] if "solid_shell" in QuadToSplineParams._default_params else (json_data["solidShell"] if "solidShell" in json_data else None)),
                        separate_by_zone if separate_by_zone is not None else ( QuadToSplineParams._default_params["separate_by_zone"] if "separate_by_zone" in QuadToSplineParams._default_params else (json_data["separateByZone"] if "separateByZone" in json_data else None)),
                        zone_name_shell_thickness_pairs if zone_name_shell_thickness_pairs is not None else ( QuadToSplineParams._default_params["zone_name_shell_thickness_pairs"] if "zone_name_shell_thickness_pairs" in QuadToSplineParams._default_params else (json_data["zoneNameShellThicknessPairs"] if "zoneNameShellThicknessPairs" in json_data else None)),
                        continuity if continuity is not None else ( QuadToSplineParams._default_params["continuity"] if "continuity" in QuadToSplineParams._default_params else SplineContinuityType(json_data["continuity"] if "continuity" in json_data else None)),
                        project_on_geometry if project_on_geometry is not None else ( QuadToSplineParams._default_params["project_on_geometry"] if "project_on_geometry" in QuadToSplineParams._default_params else (json_data["projectOnGeometry"] if "projectOnGeometry" in json_data else None)),
                        use_projection_scope if use_projection_scope is not None else ( QuadToSplineParams._default_params["use_projection_scope"] if "use_projection_scope" in QuadToSplineParams._default_params else (json_data["useProjectionScope"] if "useProjectionScope" in json_data else None)),
                        projection_scope if projection_scope is not None else ( QuadToSplineParams._default_params["projection_scope"] if "projection_scope" in QuadToSplineParams._default_params else ScopeDefinition(model = model, json_data = (json_data["projectionScope"] if "projectionScope" in json_data else None))))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            feature_capture_type: SplineFeatureCaptureType = None,
            feature_angle: float = None,
            corner_angle: float = None,
            shell_thickness: float = None,
            solid_shell: bool = None,
            separate_by_zone: bool = None,
            zone_name_shell_thickness_pairs: Dict[str, Union[str, int, float, bool]] = None,
            continuity: SplineContinuityType = None,
            project_on_geometry: bool = None,
            use_projection_scope: bool = None,
            projection_scope: ScopeDefinition = None):
        """Set the default values of the ``QuadToSplineParams`` object.

        Parameters
        ----------
        feature_capture_type: SplineFeatureCaptureType, optional
            Feature capture options.
        feature_angle: float, optional
            Angle to capture the feature.
        corner_angle: float, optional
            Corner angle of the feature.
        shell_thickness: float, optional
            Thickness of shell.
        solid_shell: bool, optional
            Solid shell option. Set true to generate solid shell spline, and set false to generate surface spline.
        separate_by_zone: bool, optional
            Option to separate IGA shell regions by zone. If set to true, it creates LS-Dyna part per zone while exporting IGA .k file and if set to false, it creates a single LS-Dyna part per Prime part, irrespective of the zones.
        zone_name_shell_thickness_pairs: Dict[str, Union[str, int, float, bool]], optional
            Zone name and thickness pair list. For example, {"Zone1Name": Zone1Thickness, "Zone2Name": Zone2Thickness, ...}.
        continuity: SplineContinuityType, optional
            Spline Continuity options.
        project_on_geometry: bool, optional
            Option to project on geometry.
        use_projection_scope: bool, optional
            Option to use projection scope.
        projection_scope: ScopeDefinition, optional
            Scope to evaluate entities for projection.
        """
        args = locals()
        [QuadToSplineParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``QuadToSplineParams`` object.

        Examples
        --------
        >>> QuadToSplineParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in QuadToSplineParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._feature_capture_type is not None:
            json_data["featureCaptureType"] = self._feature_capture_type
        if self._feature_angle is not None:
            json_data["featureAngle"] = self._feature_angle
        if self._corner_angle is not None:
            json_data["cornerAngle"] = self._corner_angle
        if self._shell_thickness is not None:
            json_data["shellThickness"] = self._shell_thickness
        if self._solid_shell is not None:
            json_data["solidShell"] = self._solid_shell
        if self._separate_by_zone is not None:
            json_data["separateByZone"] = self._separate_by_zone
        if self._zone_name_shell_thickness_pairs is not None:
            json_data["zoneNameShellThicknessPairs"] = self._zone_name_shell_thickness_pairs
        if self._continuity is not None:
            json_data["continuity"] = self._continuity
        if self._project_on_geometry is not None:
            json_data["projectOnGeometry"] = self._project_on_geometry
        if self._use_projection_scope is not None:
            json_data["useProjectionScope"] = self._use_projection_scope
        if self._projection_scope is not None:
            json_data["projectionScope"] = self._projection_scope._jsonify()
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "feature_capture_type :  %s\nfeature_angle :  %s\ncorner_angle :  %s\nshell_thickness :  %s\nsolid_shell :  %s\nseparate_by_zone :  %s\nzone_name_shell_thickness_pairs :  %s\ncontinuity :  %s\nproject_on_geometry :  %s\nuse_projection_scope :  %s\nprojection_scope :  %s" % (self._feature_capture_type, self._feature_angle, self._corner_angle, self._shell_thickness, self._solid_shell, self._separate_by_zone, self._zone_name_shell_thickness_pairs, self._continuity, self._project_on_geometry, self._use_projection_scope, '{ ' + str(self._projection_scope) + ' }')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def feature_capture_type(self) -> SplineFeatureCaptureType:
        """Feature capture options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._feature_capture_type

    @feature_capture_type.setter
    def feature_capture_type(self, value: SplineFeatureCaptureType):
        self._feature_capture_type = value

    @property
    def feature_angle(self) -> float:
        """Angle to capture the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._feature_angle

    @feature_angle.setter
    def feature_angle(self, value: float):
        self._feature_angle = value

    @property
    def corner_angle(self) -> float:
        """Corner angle of the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._corner_angle

    @corner_angle.setter
    def corner_angle(self, value: float):
        self._corner_angle = value

    @property
    def shell_thickness(self) -> float:
        """Thickness of shell.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._shell_thickness

    @shell_thickness.setter
    def shell_thickness(self, value: float):
        self._shell_thickness = value

    @property
    def solid_shell(self) -> bool:
        """Solid shell option. Set true to generate solid shell spline, and set false to generate surface spline.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._solid_shell

    @solid_shell.setter
    def solid_shell(self, value: bool):
        self._solid_shell = value

    @property
    def separate_by_zone(self) -> bool:
        """Option to separate IGA shell regions by zone. If set to true, it creates LS-Dyna part per zone while exporting IGA .k file and if set to false, it creates a single LS-Dyna part per Prime part, irrespective of the zones.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._separate_by_zone

    @separate_by_zone.setter
    def separate_by_zone(self, value: bool):
        self._separate_by_zone = value

    @property
    def zone_name_shell_thickness_pairs(self) -> Dict[str, Union[str, int, float, bool]]:
        """Zone name and thickness pair list. For example, {"Zone1Name": Zone1Thickness, "Zone2Name": Zone2Thickness, ...}.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._zone_name_shell_thickness_pairs

    @zone_name_shell_thickness_pairs.setter
    def zone_name_shell_thickness_pairs(self, value: Dict[str, Union[str, int, float, bool]]):
        self._zone_name_shell_thickness_pairs = value

    @property
    def continuity(self) -> SplineContinuityType:
        """Spline Continuity options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._continuity

    @continuity.setter
    def continuity(self, value: SplineContinuityType):
        self._continuity = value

    @property
    def project_on_geometry(self) -> bool:
        """Option to project on geometry.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._project_on_geometry

    @project_on_geometry.setter
    def project_on_geometry(self, value: bool):
        self._project_on_geometry = value

    @property
    def use_projection_scope(self) -> bool:
        """Option to use projection scope.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._use_projection_scope

    @use_projection_scope.setter
    def use_projection_scope(self, value: bool):
        self._use_projection_scope = value

    @property
    def projection_scope(self) -> ScopeDefinition:
        """Scope to evaluate entities for projection.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._projection_scope

    @projection_scope.setter
    def projection_scope(self, value: ScopeDefinition):
        self._projection_scope = value

class HexToSplineParams(CoreObject):
    """Parameters to control conversion of hex mesh to spline.

    Parameters
    ----------
    model: Model
        Model to create a ``HexToSplineParams`` object with default parameters.
    feature_capture_type: SplineFeatureCaptureType, optional
        Feature capture options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    feature_angle: float, optional
        Angle to capture the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    corner_angle: float, optional
        Corner angle of the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    continuity: SplineContinuityType, optional
        Spline Continuity options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    project_on_geometry: bool, optional
        Option to project on geometry.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    use_projection_scope: bool, optional
        Option to use projection scope.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    projection_scope: ScopeDefinition, optional
        Scope to evaluate entities for projection.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``HexToSplineParams`` object with provided parameters.

    Examples
    --------
    >>> hex_to_spline_params = prime.HexToSplineParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            feature_capture_type: SplineFeatureCaptureType,
            feature_angle: float,
            corner_angle: float,
            continuity: SplineContinuityType,
            project_on_geometry: bool,
            use_projection_scope: bool,
            projection_scope: ScopeDefinition):
        self._feature_capture_type = SplineFeatureCaptureType(feature_capture_type)
        self._feature_angle = feature_angle
        self._corner_angle = corner_angle
        self._continuity = SplineContinuityType(continuity)
        self._project_on_geometry = project_on_geometry
        self._use_projection_scope = use_projection_scope
        self._projection_scope = projection_scope

    def __init__(
            self,
            model: CommunicationManager=None,
            feature_capture_type: SplineFeatureCaptureType = None,
            feature_angle: float = None,
            corner_angle: float = None,
            continuity: SplineContinuityType = None,
            project_on_geometry: bool = None,
            use_projection_scope: bool = None,
            projection_scope: ScopeDefinition = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``HexToSplineParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``HexToSplineParams`` object with default parameters.
        feature_capture_type: SplineFeatureCaptureType, optional
            Feature capture options.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        feature_angle: float, optional
            Angle to capture the feature.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        corner_angle: float, optional
            Corner angle of the feature.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        continuity: SplineContinuityType, optional
            Spline Continuity options.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        project_on_geometry: bool, optional
            Option to project on geometry.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        use_projection_scope: bool, optional
            Option to use projection scope.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        projection_scope: ScopeDefinition, optional
            Scope to evaluate entities for projection.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``HexToSplineParams`` object with provided parameters.

        Examples
        --------
        >>> hex_to_spline_params = prime.HexToSplineParams(model = model)
        """
        if json_data:
            self.__initialize(
                SplineFeatureCaptureType(json_data["featureCaptureType"] if "featureCaptureType" in json_data else None),
                json_data["featureAngle"] if "featureAngle" in json_data else None,
                json_data["cornerAngle"] if "cornerAngle" in json_data else None,
                SplineContinuityType(json_data["continuity"] if "continuity" in json_data else None),
                json_data["projectOnGeometry"] if "projectOnGeometry" in json_data else None,
                json_data["useProjectionScope"] if "useProjectionScope" in json_data else None,
                ScopeDefinition(model = model, json_data = json_data["projectionScope"] if "projectionScope" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [feature_capture_type, feature_angle, corner_angle, continuity, project_on_geometry, use_projection_scope, projection_scope])
            if all_field_specified:
                self.__initialize(
                    feature_capture_type,
                    feature_angle,
                    corner_angle,
                    continuity,
                    project_on_geometry,
                    use_projection_scope,
                    projection_scope)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "HexToSplineParams")
                    json_data = param_json["HexToSplineParams"] if "HexToSplineParams" in param_json else {}
                    self.__initialize(
                        feature_capture_type if feature_capture_type is not None else ( HexToSplineParams._default_params["feature_capture_type"] if "feature_capture_type" in HexToSplineParams._default_params else SplineFeatureCaptureType(json_data["featureCaptureType"] if "featureCaptureType" in json_data else None)),
                        feature_angle if feature_angle is not None else ( HexToSplineParams._default_params["feature_angle"] if "feature_angle" in HexToSplineParams._default_params else (json_data["featureAngle"] if "featureAngle" in json_data else None)),
                        corner_angle if corner_angle is not None else ( HexToSplineParams._default_params["corner_angle"] if "corner_angle" in HexToSplineParams._default_params else (json_data["cornerAngle"] if "cornerAngle" in json_data else None)),
                        continuity if continuity is not None else ( HexToSplineParams._default_params["continuity"] if "continuity" in HexToSplineParams._default_params else SplineContinuityType(json_data["continuity"] if "continuity" in json_data else None)),
                        project_on_geometry if project_on_geometry is not None else ( HexToSplineParams._default_params["project_on_geometry"] if "project_on_geometry" in HexToSplineParams._default_params else (json_data["projectOnGeometry"] if "projectOnGeometry" in json_data else None)),
                        use_projection_scope if use_projection_scope is not None else ( HexToSplineParams._default_params["use_projection_scope"] if "use_projection_scope" in HexToSplineParams._default_params else (json_data["useProjectionScope"] if "useProjectionScope" in json_data else None)),
                        projection_scope if projection_scope is not None else ( HexToSplineParams._default_params["projection_scope"] if "projection_scope" in HexToSplineParams._default_params else ScopeDefinition(model = model, json_data = (json_data["projectionScope"] if "projectionScope" in json_data else None))))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            feature_capture_type: SplineFeatureCaptureType = None,
            feature_angle: float = None,
            corner_angle: float = None,
            continuity: SplineContinuityType = None,
            project_on_geometry: bool = None,
            use_projection_scope: bool = None,
            projection_scope: ScopeDefinition = None):
        """Set the default values of the ``HexToSplineParams`` object.

        Parameters
        ----------
        feature_capture_type: SplineFeatureCaptureType, optional
            Feature capture options.
        feature_angle: float, optional
            Angle to capture the feature.
        corner_angle: float, optional
            Corner angle of the feature.
        continuity: SplineContinuityType, optional
            Spline Continuity options.
        project_on_geometry: bool, optional
            Option to project on geometry.
        use_projection_scope: bool, optional
            Option to use projection scope.
        projection_scope: ScopeDefinition, optional
            Scope to evaluate entities for projection.
        """
        args = locals()
        [HexToSplineParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``HexToSplineParams`` object.

        Examples
        --------
        >>> HexToSplineParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in HexToSplineParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._feature_capture_type is not None:
            json_data["featureCaptureType"] = self._feature_capture_type
        if self._feature_angle is not None:
            json_data["featureAngle"] = self._feature_angle
        if self._corner_angle is not None:
            json_data["cornerAngle"] = self._corner_angle
        if self._continuity is not None:
            json_data["continuity"] = self._continuity
        if self._project_on_geometry is not None:
            json_data["projectOnGeometry"] = self._project_on_geometry
        if self._use_projection_scope is not None:
            json_data["useProjectionScope"] = self._use_projection_scope
        if self._projection_scope is not None:
            json_data["projectionScope"] = self._projection_scope._jsonify()
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "feature_capture_type :  %s\nfeature_angle :  %s\ncorner_angle :  %s\ncontinuity :  %s\nproject_on_geometry :  %s\nuse_projection_scope :  %s\nprojection_scope :  %s" % (self._feature_capture_type, self._feature_angle, self._corner_angle, self._continuity, self._project_on_geometry, self._use_projection_scope, '{ ' + str(self._projection_scope) + ' }')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def feature_capture_type(self) -> SplineFeatureCaptureType:
        """Feature capture options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._feature_capture_type

    @feature_capture_type.setter
    def feature_capture_type(self, value: SplineFeatureCaptureType):
        self._feature_capture_type = value

    @property
    def feature_angle(self) -> float:
        """Angle to capture the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._feature_angle

    @feature_angle.setter
    def feature_angle(self, value: float):
        self._feature_angle = value

    @property
    def corner_angle(self) -> float:
        """Corner angle of the feature.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._corner_angle

    @corner_angle.setter
    def corner_angle(self, value: float):
        self._corner_angle = value

    @property
    def continuity(self) -> SplineContinuityType:
        """Spline Continuity options.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._continuity

    @continuity.setter
    def continuity(self, value: SplineContinuityType):
        self._continuity = value

    @property
    def project_on_geometry(self) -> bool:
        """Option to project on geometry.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._project_on_geometry

    @project_on_geometry.setter
    def project_on_geometry(self, value: bool):
        self._project_on_geometry = value

    @property
    def use_projection_scope(self) -> bool:
        """Option to use projection scope.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._use_projection_scope

    @use_projection_scope.setter
    def use_projection_scope(self, value: bool):
        self._use_projection_scope = value

    @property
    def projection_scope(self) -> ScopeDefinition:
        """Scope to evaluate entities for projection.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._projection_scope

    @projection_scope.setter
    def projection_scope(self, value: ScopeDefinition):
        self._projection_scope = value

class RefineSplineParams(CoreObject):
    """Spline refinement parameters.

    Parameters
    ----------
    model: Model
        Model to create a ``RefineSplineParams`` object with default parameters.
    refine_flag_u: bool, optional
        Indicates whether refinement is applied in u direction.
    refine_flag_v: bool, optional
        Indicates whether refinement is applied in v direction.
    refine_flag_w: bool, optional
        Indicates whether refinement is applied in w direction.
    spline_refinement_type: SplineRefinementType, optional
        Type of spline refinement. Currently, supports h-refinement and p-refinement.
    json_data: dict, optional
        JSON dictionary to create a ``RefineSplineParams`` object with provided parameters.

    Examples
    --------
    >>> refine_spline_params = prime.RefineSplineParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            refine_flag_u: bool,
            refine_flag_v: bool,
            refine_flag_w: bool,
            spline_refinement_type: SplineRefinementType):
        self._refine_flag_u = refine_flag_u
        self._refine_flag_v = refine_flag_v
        self._refine_flag_w = refine_flag_w
        self._spline_refinement_type = SplineRefinementType(spline_refinement_type)

    def __init__(
            self,
            model: CommunicationManager=None,
            refine_flag_u: bool = None,
            refine_flag_v: bool = None,
            refine_flag_w: bool = None,
            spline_refinement_type: SplineRefinementType = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``RefineSplineParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``RefineSplineParams`` object with default parameters.
        refine_flag_u: bool, optional
            Indicates whether refinement is applied in u direction.
        refine_flag_v: bool, optional
            Indicates whether refinement is applied in v direction.
        refine_flag_w: bool, optional
            Indicates whether refinement is applied in w direction.
        spline_refinement_type: SplineRefinementType, optional
            Type of spline refinement. Currently, supports h-refinement and p-refinement.
        json_data: dict, optional
            JSON dictionary to create a ``RefineSplineParams`` object with provided parameters.

        Examples
        --------
        >>> refine_spline_params = prime.RefineSplineParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["refineFlagU"] if "refineFlagU" in json_data else None,
                json_data["refineFlagV"] if "refineFlagV" in json_data else None,
                json_data["refineFlagW"] if "refineFlagW" in json_data else None,
                SplineRefinementType(json_data["splineRefinementType"] if "splineRefinementType" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [refine_flag_u, refine_flag_v, refine_flag_w, spline_refinement_type])
            if all_field_specified:
                self.__initialize(
                    refine_flag_u,
                    refine_flag_v,
                    refine_flag_w,
                    spline_refinement_type)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "RefineSplineParams")
                    json_data = param_json["RefineSplineParams"] if "RefineSplineParams" in param_json else {}
                    self.__initialize(
                        refine_flag_u if refine_flag_u is not None else ( RefineSplineParams._default_params["refine_flag_u"] if "refine_flag_u" in RefineSplineParams._default_params else (json_data["refineFlagU"] if "refineFlagU" in json_data else None)),
                        refine_flag_v if refine_flag_v is not None else ( RefineSplineParams._default_params["refine_flag_v"] if "refine_flag_v" in RefineSplineParams._default_params else (json_data["refineFlagV"] if "refineFlagV" in json_data else None)),
                        refine_flag_w if refine_flag_w is not None else ( RefineSplineParams._default_params["refine_flag_w"] if "refine_flag_w" in RefineSplineParams._default_params else (json_data["refineFlagW"] if "refineFlagW" in json_data else None)),
                        spline_refinement_type if spline_refinement_type is not None else ( RefineSplineParams._default_params["spline_refinement_type"] if "spline_refinement_type" in RefineSplineParams._default_params else SplineRefinementType(json_data["splineRefinementType"] if "splineRefinementType" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            refine_flag_u: bool = None,
            refine_flag_v: bool = None,
            refine_flag_w: bool = None,
            spline_refinement_type: SplineRefinementType = None):
        """Set the default values of the ``RefineSplineParams`` object.

        Parameters
        ----------
        refine_flag_u: bool, optional
            Indicates whether refinement is applied in u direction.
        refine_flag_v: bool, optional
            Indicates whether refinement is applied in v direction.
        refine_flag_w: bool, optional
            Indicates whether refinement is applied in w direction.
        spline_refinement_type: SplineRefinementType, optional
            Type of spline refinement. Currently, supports h-refinement and p-refinement.
        """
        args = locals()
        [RefineSplineParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``RefineSplineParams`` object.

        Examples
        --------
        >>> RefineSplineParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in RefineSplineParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._refine_flag_u is not None:
            json_data["refineFlagU"] = self._refine_flag_u
        if self._refine_flag_v is not None:
            json_data["refineFlagV"] = self._refine_flag_v
        if self._refine_flag_w is not None:
            json_data["refineFlagW"] = self._refine_flag_w
        if self._spline_refinement_type is not None:
            json_data["splineRefinementType"] = self._spline_refinement_type
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "refine_flag_u :  %s\nrefine_flag_v :  %s\nrefine_flag_w :  %s\nspline_refinement_type :  %s" % (self._refine_flag_u, self._refine_flag_v, self._refine_flag_w, self._spline_refinement_type)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def refine_flag_u(self) -> bool:
        """Indicates whether refinement is applied in u direction.
        """
        return self._refine_flag_u

    @refine_flag_u.setter
    def refine_flag_u(self, value: bool):
        self._refine_flag_u = value

    @property
    def refine_flag_v(self) -> bool:
        """Indicates whether refinement is applied in v direction.
        """
        return self._refine_flag_v

    @refine_flag_v.setter
    def refine_flag_v(self, value: bool):
        self._refine_flag_v = value

    @property
    def refine_flag_w(self) -> bool:
        """Indicates whether refinement is applied in w direction.
        """
        return self._refine_flag_w

    @refine_flag_w.setter
    def refine_flag_w(self, value: bool):
        self._refine_flag_w = value

    @property
    def spline_refinement_type(self) -> SplineRefinementType:
        """Type of spline refinement. Currently, supports h-refinement and p-refinement.
        """
        return self._spline_refinement_type

    @spline_refinement_type.setter
    def spline_refinement_type(self, value: SplineRefinementType):
        self._spline_refinement_type = value



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class MultiZoneControl(CoreObject):
    """MultiZoneControl provides a way to gather all the information required for MultiZone meshing.

    The MultiZone meshing provides hex meshing capabilities.
    Different type of mesh can be generated using MultiZoneControl. For example, sweep mesh, map mesh and edge biased mesh.

    Parameters
    ----------
    model : Model
        Server model to create MultiZoneControl object.
    id : int
        Id of the MultiZoneControl.
    object_id : int
        Object id of the MultiZoneControl.
    name : str
        Name of the MultiZoneControl.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize MultiZoneControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def set_volume_scope(self, scope_info : ScopeDefinition):
        """ Sets the scope for volume in terms of topovolumes.The topovolumes given by the scope are imported by MultiZone to generate MultiZone mesh on it.


        Parameters
        ----------
        scope_info : ScopeDefinition
            Scoped topovolumes.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> multizone_control.set_volume_scope(scope_info)

        """
        if not isinstance(scope_info, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'scope_info'. Valid argument type is ScopeDefinition.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetVolumeScope"
        self._model._print_beta_api_warning("set_volume_scope")
        self._model._print_logs_before_command("set_volume_scope", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_volume_scope")

    def set_surface_scope(self, scope_info : ScopeDefinition):
        """ Sets the scope for surface in terms of topofaces.


        Parameters
        ----------
        scope_info : ScopeDefinition
            Scoped topofaces.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> multizone_control.set_surface_scope(scope_info)

        """
        if not isinstance(scope_info, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'scope_info'. Valid argument type is ScopeDefinition.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetSurfaceScope"
        self._model._print_beta_api_warning("set_surface_scope")
        self._model._print_logs_before_command("set_surface_scope", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_surface_scope")

    def set_map_mesh_params(self, scope_info : MultiZoneMapMeshParams):
        """ Set the parameters for map meshing in terms of topofaces during MultiZone mesh.


        Parameters
        ----------
        scope_info : MultiZoneMapMeshParams
            Scoped topofaces to be map meshed.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> multizone_control.set_map_mesh_params(scope_info)

        """
        if not isinstance(scope_info, MultiZoneMapMeshParams):
            raise TypeError("Invalid argument type passed for 'scope_info'. Valid argument type is MultiZoneMapMeshParams.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetMapMeshParams"
        self._model._print_beta_api_warning("set_map_mesh_params")
        self._model._print_logs_before_command("set_map_mesh_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_map_mesh_params")

    def set_sweep_mesh_params(self, scope_info : MultiZoneSweepMeshParams):
        """ Set the parameters for sweep meshing in terms of topofaces during MultiZone mesh.


        Parameters
        ----------
        scope_info : MultiZoneSweepMeshParams
            Information required for sweep meshing.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> multizone_control.set_sweep_mesh_params(scope_info)

        """
        if not isinstance(scope_info, MultiZoneSweepMeshParams):
            raise TypeError("Invalid argument type passed for 'scope_info'. Valid argument type is MultiZoneSweepMeshParams.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetSweepMeshParams"
        self._model._print_beta_api_warning("set_sweep_mesh_params")
        self._model._print_logs_before_command("set_sweep_mesh_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_sweep_mesh_params")

    def set_edge_biasing_params(self, scope_info : MultiZoneEdgeBiasingParams):
        """ Sets the parameters for edge biasing in terms of topoedges and topofaces during MultiZone mesh.


        Parameters
        ----------
        scope_info : MultiZoneEdgeBiasingParams
            Information for edge biasing in MultiZone meshing.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> multizone_control.set_edge_biasing_params(scope_info)

        """
        if not isinstance(scope_info, MultiZoneEdgeBiasingParams):
            raise TypeError("Invalid argument type passed for 'scope_info'. Valid argument type is MultiZoneEdgeBiasingParams.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetEdgeBiasingParams"
        self._model._print_beta_api_warning("set_edge_biasing_params")
        self._model._print_logs_before_command("set_edge_biasing_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_edge_biasing_params")

    def set_multi_zone_sizing_params(self, params : MultiZoneSizingParams):
        """ Sets the MultiZone parameters.


        Parameters
        ----------
        params : MultiZoneSizingParams
            Parameters required for MultiZone mesh.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> multizone_control.set_multi_zone_params(params)

        """
        if not isinstance(params, MultiZoneSizingParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is MultiZoneSizingParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetMultiZoneSizingParams"
        self._model._print_beta_api_warning("set_multi_zone_sizing_params")
        self._model._print_logs_before_command("set_multi_zone_sizing_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_multi_zone_sizing_params")

    @property
    def id(self):
        """ Get the id of MultiZoneControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of MultiZoneControl."""
        return self._name



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class CollapseParams(CoreObject):
    """Parameters to collapse face elements.

    Parameters
    ----------
    model: Model
        Model to create a ``CollapseParams`` object with default parameters.
    feature_type: SurfaceFeatureType, optional
        Feature type to be preserved when performing collapse.
    collapse_ratio: float, optional
        Maximum ratio of shortest face edge length to longest face edge length.
    preserve_quality: bool, optional
        Option to preserve quality of neighboring triangles when performing collapse.Collapse may lead to quality deterioration beyond target skewness. Such collapse is prevented, when the option is enabled.
    target_skewness: float, optional
        Skewness limit used as target to preserve quality. Better quality elements are skipped for collapse.
    keep_edge_connectivity: bool, optional
        Option to keep edge connectivity. The default value is false. When keep edge connectivity is true, edges that were connected to collapsed triangles will be connected to different triangles. Otherwise, edges will be disconnected from faces.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``CollapseParams`` object with provided parameters.

    Examples
    --------
    >>> collapse_params = prime.CollapseParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            feature_type: SurfaceFeatureType,
            collapse_ratio: float,
            preserve_quality: bool,
            target_skewness: float,
            keep_edge_connectivity: bool):
        self._feature_type = SurfaceFeatureType(feature_type)
        self._collapse_ratio = collapse_ratio
        self._preserve_quality = preserve_quality
        self._target_skewness = target_skewness
        self._keep_edge_connectivity = keep_edge_connectivity

    def __init__(
            self,
            model: CommunicationManager=None,
            feature_type: SurfaceFeatureType = None,
            collapse_ratio: float = None,
            preserve_quality: bool = None,
            target_skewness: float = None,
            keep_edge_connectivity: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``CollapseParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``CollapseParams`` object with default parameters.
        feature_type: SurfaceFeatureType, optional
            Feature type to be preserved when performing collapse.
        collapse_ratio: float, optional
            Maximum ratio of shortest face edge length to longest face edge length.
        preserve_quality: bool, optional
            Option to preserve quality of neighboring triangles when performing collapse.Collapse may lead to quality deterioration beyond target skewness. Such collapse is prevented, when the option is enabled.
        target_skewness: float, optional
            Skewness limit used as target to preserve quality. Better quality elements are skipped for collapse.
        keep_edge_connectivity: bool, optional
            Option to keep edge connectivity. The default value is false. When keep edge connectivity is true, edges that were connected to collapsed triangles will be connected to different triangles. Otherwise, edges will be disconnected from faces.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``CollapseParams`` object with provided parameters.

        Examples
        --------
        >>> collapse_params = prime.CollapseParams(model = model)
        """
        if json_data:
            self.__initialize(
                SurfaceFeatureType(json_data["featureType"] if "featureType" in json_data else None),
                json_data["collapseRatio"] if "collapseRatio" in json_data else None,
                json_data["preserveQuality"] if "preserveQuality" in json_data else None,
                json_data["targetSkewness"] if "targetSkewness" in json_data else None,
                json_data["keepEdgeConnectivity"] if "keepEdgeConnectivity" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [feature_type, collapse_ratio, preserve_quality, target_skewness, keep_edge_connectivity])
            if all_field_specified:
                self.__initialize(
                    feature_type,
                    collapse_ratio,
                    preserve_quality,
                    target_skewness,
                    keep_edge_connectivity)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "CollapseParams")
                    json_data = param_json["CollapseParams"] if "CollapseParams" in param_json else {}
                    self.__initialize(
                        feature_type if feature_type is not None else ( CollapseParams._default_params["feature_type"] if "feature_type" in CollapseParams._default_params else SurfaceFeatureType(json_data["featureType"] if "featureType" in json_data else None)),
                        collapse_ratio if collapse_ratio is not None else ( CollapseParams._default_params["collapse_ratio"] if "collapse_ratio" in CollapseParams._default_params else (json_data["collapseRatio"] if "collapseRatio" in json_data else None)),
                        preserve_quality if preserve_quality is not None else ( CollapseParams._default_params["preserve_quality"] if "preserve_quality" in CollapseParams._default_params else (json_data["preserveQuality"] if "preserveQuality" in json_data else None)),
                        target_skewness if target_skewness is not None else ( CollapseParams._default_params["target_skewness"] if "target_skewness" in CollapseParams._default_params else (json_data["targetSkewness"] if "targetSkewness" in json_data else None)),
                        keep_edge_connectivity if keep_edge_connectivity is not None else ( CollapseParams._default_params["keep_edge_connectivity"] if "keep_edge_connectivity" in CollapseParams._default_params else (json_data["keepEdgeConnectivity"] if "keepEdgeConnectivity" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            feature_type: SurfaceFeatureType = None,
            collapse_ratio: float = None,
            preserve_quality: bool = None,
            target_skewness: float = None,
            keep_edge_connectivity: bool = None):
        """Set the default values of the ``CollapseParams`` object.

        Parameters
        ----------
        feature_type: SurfaceFeatureType, optional
            Feature type to be preserved when performing collapse.
        collapse_ratio: float, optional
            Maximum ratio of shortest face edge length to longest face edge length.
        preserve_quality: bool, optional
            Option to preserve quality of neighboring triangles when performing collapse.Collapse may lead to quality deterioration beyond target skewness. Such collapse is prevented, when the option is enabled.
        target_skewness: float, optional
            Skewness limit used as target to preserve quality. Better quality elements are skipped for collapse.
        keep_edge_connectivity: bool, optional
            Option to keep edge connectivity. The default value is false. When keep edge connectivity is true, edges that were connected to collapsed triangles will be connected to different triangles. Otherwise, edges will be disconnected from faces.
        """
        args = locals()
        [CollapseParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``CollapseParams`` object.

        Examples
        --------
        >>> CollapseParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CollapseParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._feature_type is not None:
            json_data["featureType"] = self._feature_type
        if self._collapse_ratio is not None:
            json_data["collapseRatio"] = self._collapse_ratio
        if self._preserve_quality is not None:
            json_data["preserveQuality"] = self._preserve_quality
        if self._target_skewness is not None:
            json_data["targetSkewness"] = self._target_skewness
        if self._keep_edge_connectivity is not None:
            json_data["keepEdgeConnectivity"] = self._keep_edge_connectivity
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "feature_type :  %s\ncollapse_ratio :  %s\npreserve_quality :  %s\ntarget_skewness :  %s\nkeep_edge_connectivity :  %s" % (self._feature_type, self._collapse_ratio, self._preserve_quality, self._target_skewness, self._keep_edge_connectivity)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def feature_type(self) -> SurfaceFeatureType:
        """Feature type to be preserved when performing collapse.
        """
        return self._feature_type

    @feature_type.setter
    def feature_type(self, value: SurfaceFeatureType):
        self._feature_type = value

    @property
    def collapse_ratio(self) -> float:
        """Maximum ratio of shortest face edge length to longest face edge length.
        """
        return self._collapse_ratio

    @collapse_ratio.setter
    def collapse_ratio(self, value: float):
        self._collapse_ratio = value

    @property
    def preserve_quality(self) -> bool:
        """Option to preserve quality of neighboring triangles when performing collapse.Collapse may lead to quality deterioration beyond target skewness. Such collapse is prevented, when the option is enabled.
        """
        return self._preserve_quality

    @preserve_quality.setter
    def preserve_quality(self, value: bool):
        self._preserve_quality = value

    @property
    def target_skewness(self) -> float:
        """Skewness limit used as target to preserve quality. Better quality elements are skipped for collapse.
        """
        return self._target_skewness

    @target_skewness.setter
    def target_skewness(self, value: float):
        self._target_skewness = value

    @property
    def keep_edge_connectivity(self) -> bool:
        """Option to keep edge connectivity. The default value is false. When keep edge connectivity is true, edges that were connected to collapsed triangles will be connected to different triangles. Otherwise, edges will be disconnected from faces.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._keep_edge_connectivity

    @keep_edge_connectivity.setter
    def keep_edge_connectivity(self, value: bool):
        self._keep_edge_connectivity = value

class CollapseResults(CoreObject):
    """Results associated with collapse face elements.

    Parameters
    ----------
    model: Model
        Model to create a ``CollapseResults`` object with default parameters.
    error_code: ErrorCode, optional
        Error code associated with the failure of operation.
    n_collapsed: int, optional
        Number of face elements collapsed.
    n_splits: int, optional
        Number of face elements split.
    json_data: dict, optional
        JSON dictionary to create a ``CollapseResults`` object with provided parameters.

    Examples
    --------
    >>> collapse_results = prime.CollapseResults(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            n_collapsed: int,
            n_splits: int):
        self._error_code = ErrorCode(error_code)
        self._n_collapsed = n_collapsed
        self._n_splits = n_splits

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            n_collapsed: int = None,
            n_splits: int = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``CollapseResults`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``CollapseResults`` object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        n_collapsed: int, optional
            Number of face elements collapsed.
        n_splits: int, optional
            Number of face elements split.
        json_data: dict, optional
            JSON dictionary to create a ``CollapseResults`` object with provided parameters.

        Examples
        --------
        >>> collapse_results = prime.CollapseResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["nCollapsed"] if "nCollapsed" in json_data else None,
                json_data["nSplits"] if "nSplits" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, n_collapsed, n_splits])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    n_collapsed,
                    n_splits)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "CollapseResults")
                    json_data = param_json["CollapseResults"] if "CollapseResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( CollapseResults._default_params["error_code"] if "error_code" in CollapseResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        n_collapsed if n_collapsed is not None else ( CollapseResults._default_params["n_collapsed"] if "n_collapsed" in CollapseResults._default_params else (json_data["nCollapsed"] if "nCollapsed" in json_data else None)),
                        n_splits if n_splits is not None else ( CollapseResults._default_params["n_splits"] if "n_splits" in CollapseResults._default_params else (json_data["nSplits"] if "nSplits" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            n_collapsed: int = None,
            n_splits: int = None):
        """Set the default values of the ``CollapseResults`` object.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        n_collapsed: int, optional
            Number of face elements collapsed.
        n_splits: int, optional
            Number of face elements split.
        """
        args = locals()
        [CollapseResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``CollapseResults`` object.

        Examples
        --------
        >>> CollapseResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CollapseResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._n_collapsed is not None:
            json_data["nCollapsed"] = self._n_collapsed
        if self._n_splits is not None:
            json_data["nSplits"] = self._n_splits
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nn_collapsed :  %s\nn_splits :  %s" % (self._error_code, self._n_collapsed, self._n_splits)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def n_collapsed(self) -> int:
        """Number of face elements collapsed.
        """
        return self._n_collapsed

    @n_collapsed.setter
    def n_collapsed(self, value: int):
        self._n_collapsed = value

    @property
    def n_splits(self) -> int:
        """Number of face elements split.
        """
        return self._n_splits

    @n_splits.setter
    def n_splits(self, value: int):
        self._n_splits = value



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Module for the plotter."""
import enum
import warnings
from typing import Any, Dict, List, Optional

import numpy as np
from ansys.tools.visualization_interface import Plotter
from ansys.tools.visualization_interface.backends.pyvista import PyVistaBackend

import ansys.meshing.prime as prime
from ansys.meshing.prime.core.mesh import DisplayMeshInfo
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.graphics.widgets.color_by_type import ColorByTypeWidget
from ansys.meshing.prime.graphics.widgets.hide_picked import HidePicked
from ansys.meshing.prime.graphics.widgets.picked_info import PickedInfo
from ansys.meshing.prime.graphics.widgets.toggle_edges import ToggleEdges

color_matrix = np.array(
    [
        [155, 186, 126],
        [242, 236, 175],
        [255, 187, 131],
        [194, 187, 97],
        [159, 131, 169],
        [157, 190, 139],
        [233, 218, 158],
        [254, 252, 196],
        [246, 210, 148],
        [215, 208, 198],
        [196, 235, 145],
    ]
)


class ColorByType(enum.IntEnum):
    """Contains the zone types to display."""

    ZONE = 0
    ZONELET = 1
    PART = 2


class PrimePlotter(Plotter):
    """Create a plotter for PyPrimeMesh models.

    This plotter is a wrapper around the PyAnsys generic plotter
    with additional functionality for PyPrimeMesh.

    Parameters
    ----------
    use_trame : Optional[bool], default: None
        Whether to use the Trame visualizer.
    allow_picking : Optional[bool], default: True.
        Whether to allow picking.
    """

    def __init__(
        self, use_trame: Optional[bool] = None, allow_picking: Optional[bool] = True
    ) -> None:
        """Initialize the widget."""
        self._backend = PyVistaBackend(use_trame=use_trame, allow_picking=allow_picking)
        super().__init__(backend=self._backend)

        # info of the actor to pass to picked info widget
        self._info_actor_map = {}
        self._backend.add_widget(ToggleEdges(self))
        self._backend.add_widget(ColorByTypeWidget(self))
        self._backend.add_widget(HidePicked(self))
        self._backend.add_widget(PickedInfo(self))

    @property
    def info_actor_map(self) -> Dict:
        """Get the information actor map for the selected information widget.

        Returns
        -------
        Dict
            Information actor map.
        """
        return self._info_actor_map

    @info_actor_map.setter
    def info_actor_map(self, value: Dict) -> None:
        """Set the information actor map for the selected information widget.

        Parameters
        ----------
        value : Dict
            Information actor map.
        """
        self._info_actor_map = value

    def get_scalar_colors(self, mesh_info: DisplayMeshInfo) -> np.ndarray:
        """Get the scalar colors for the mesh.

        Parameters
        ----------
        mesh_info : DisplayMeshInfo
            Mesh information that generates an appropriate color.

        Returns
        -------
        np.ndarray
            Scalar colors for the mesh.
        """
        mesh_type = mesh_info.display_mesh_type
        num_colors = int(color_matrix.size / 3)
        if mesh_type == ColorByType.ZONELET:
            return color_matrix[mesh_info.id % num_colors].tolist()
        elif mesh_type == ColorByType.PART:
            return color_matrix[mesh_info.part_id % num_colors].tolist()
        else:
            return color_matrix[mesh_info.zone_id % num_colors].tolist()

    def add_model(
        self, model: Model, scope: prime.ScopeDefinition = None, update: bool = False
    ) -> None:
        """Add a Prime model to the plotter.

        Parameters
        ----------
        model : Model
            Prime model to add.
        scope : prime.ScopeDefinition, default: None
            Scope to show, if any.
        update : bool, default: False
            Whether to update the display.
        """
        if scope is None:
            model_pd = model.as_polydata(update=update)
            self.add_model_pd(model_pd)
        else:
            self.add_scope(model, scope, update=update)

    def add_model_pd(self, model_pd: Dict) -> None:
        """Add a model to the plotter.

        Parameters
        ----------
        model : Model
            Model to add to the plotter.
        """
        for part_id, part_polydata in model_pd.items():
            # proceed if scope won't be used or if the part is in the scope
            if "faces" in part_polydata.keys():
                for face_mesh_part, face_mesh_info in part_polydata["faces"]:

                    # These operations could be done downstream,
                    # but we need the actor for the picked info widget
                    colors = self.get_scalar_colors(face_mesh_info)
                    has_mesh = face_mesh_info.has_mesh
                    actor = self._backend.pv_interface.scene.add_mesh(
                        face_mesh_part.mesh, show_edges=has_mesh, color=colors, pickable=True
                    )
                    face_mesh_part.actor = actor
                    self._backend.pv_interface._object_to_actors_map[actor] = face_mesh_part
                    self._info_actor_map[actor] = face_mesh_info

            if "edges" in part_polydata.keys():
                for edge_mesh_part in part_polydata["edges"]:
                    actor = self._backend.pv_interface.scene.add_mesh(
                        edge_mesh_part.mesh,
                        # scalars="colors",
                        rgb=True,
                        pickable=False,
                        line_width=4,
                    )
                    edge_mesh_part.actor = actor
                    self._backend._object_to_actors_map[actor] = edge_mesh_part

            if "ctrlpoints" in part_polydata.keys():
                for ctrlpoint_mesh_part in part_polydata["ctrlpoints"]:
                    actor = self._backend.pv_interface.scene.add_mesh(
                        ctrlpoint_mesh_part.mesh,
                        show_edges=False,
                        # scalars="colors",
                        rgb=True,
                        pickable=False,
                        style='wireframe',
                        edge_color=[0, 0, 255],
                    )
                    ctrlpoint_mesh_part.actor = actor
                    self._backend._object_to_actors_map[actor] = ctrlpoint_mesh_part

            if "splines" in part_polydata.keys():
                for spline_mesh_part in part_polydata["splines"]:
                    actor = self._backend._pl.scene.add_mesh(
                        spline_mesh_part.mesh,
                        show_edges=False,
                        # scalars="colors",
                        rgb=True,
                        pickable=False,
                    )
                    spline_mesh_part.actor = actor
                    self._backend._object_to_actors_map[actor] = spline_mesh_part

    def add_scope(self, model: Model, scope: prime.ScopeDefinition, update: bool = False) -> None:
        """Add a scope to the plotter.

        Parameters
        ----------
        model : Model
            Model to add to the plotter.
        scope : prime.ScopeDefinition
            Scope to add to the plotter.
        update : bool, default: False
            Whether to update the display.
        """
        model_pd = model.get_scoped_polydata(scope, update=update)
        self.add_model_pd(model_pd)

    def plot_iter(
        self,
        plotting_list: List[Any],
        name_filter: str = None,
        update: bool = False,
        **plotting_options,
    ) -> None:
        """
        Add a list of any type of object to the scene.

        Allowed types are PyPrime models or any PyVista plottable object.

        Parameters
        ----------
        plotting_list : List[Any]
            List of objects to plot.
        name_filter : str, default: None
            Regular expression with the desired name or names to include in the plotter.
        update: bool, default: False
            Whether to update the display.
        **plotting_options : dict, default: None
            Keyword arguments. For allowable keyword arguments, see the
            :meth:`Plotter.add_mesh <pyvista.Plotter.add_mesh>` method.
            Options only applied to PyVista plottable objects.
        """
        for plottable_object in plotting_list:
            _ = self.plot(plottable_object, name_filter, **plotting_options)

    def plot(
        self,
        plottable_object: Any,
        scope: prime.ScopeDefinition = None,
        name_filter: str = None,
        update: bool = False,
        **plotting_options,
    ):
        """Add an object to the plotter.

        Allowed types are PyPrime models or any PyVista plottable object.

        Parameters
        ----------
        plottable_object : Any
            Object to add to the plotter.
        scope : prime.ScopeDefinition, default: None
            Scope to plot.
        name_filter : str, default: None
            Regular expression with the desired name or names to include in the plotter.
        update: bool, default: False
            Whether to update the display. Required when any mesh is updated.
        **plotting_options : dict, default: None
            Keyword arguments. For allowable keyword arguments, see the
            :meth:`Plotter.add_mesh <pyvista.Plotter.add_mesh>` method.
            Options only applied to PyVista plottable objects.


        Examples
        --------
        >>> import pyvista as pv
        >>> from ansys.meshing.prime.graphics import PrimePlotter
        >>> import ansys.meshing.prime as prime
        >>> model = prime.launch_prime().model
        >>> prime.lucid.Mesh(model).read(prime.examples.download_block_model_fmd())
        >>> scope = prime.ScopeDefinition(model, label_expression="my_group")
        >>> plotter = PrimePlotter()
        >>> # pyvista sphere with plotting options added for opacity and color
        >>> plotter.plot(plottable_object=pv.Sphere(radius=2.0), opacity=0.5, color="red")
        >>> plotter.plot(plottable_object=model, scope=scope)
        >>> plotter.show()

        """
        if isinstance(plottable_object, Model):
            self.add_model(plottable_object, scope, update=update)
        elif isinstance(plottable_object, List):
            self.plot_iter(plottable_object, name_filter, update=update, **plotting_options)
        else:
            self._backend.pv_interface.plot(plottable_object, name_filter, **plotting_options)

    def show(
        self,
        plottable_object: Any = None,
        screenshot: str = None,
        name_filter: str = None,
        scope: prime.ScopeDefinition = None,
        **plotting_options,
    ) -> None:
        """Show the plotted objects.

        Parameters
        ----------
        plottable_object : Any, default: None
            Object to show.
        screenshot : str, default: None
            Path to save a screenshot to.
        name_filter : str, default: None
            Regular expression with the desired name or names to include in the plotter.
        **plotting_options : dict, default: None
            Keyword arguments. For allowable keyword arguments, see the
            :meth:`Plotter.add_mesh <pyvista.Plotter.add_mesh>` method.
            Options only applied to PyVista plottable objects.
        """
        if plottable_object is not None:
            self.plot(plottable_object, name_filter=name_filter, scope=scope, **plotting_options)
        self._backend.show(
            plottable_object=plottable_object,
            screenshot=screenshot,
            name_filter=name_filter,
            **plotting_options,
        )


class Graphics:
    """Manages graphics in PyPrime.

    .. deprecated:: 0.6.0
        Use :class:`PrimePlotter` instead.

    Parameters
    ----------
    model : prime.Model
        Model to show.
    use_trame : bool, default: False
        Whether to use the Trame visualizer.
    """

    def __init__(self, model: prime.Model, use_trame: bool = False) -> None:
        """Initialize graphics."""
        self.model = model
        self.use_trame = use_trame
        warnings.warn(
            "DeprecationWarning: The `Graphics` class is deprecated. "
            + "Use the `PrimePlotter` class instead."
        )

    def __call__(
        self,
        parts: List = None,
        update: bool = True,
        spline: bool = False,
        scope: prime.ScopeDefinition = None,
    ) -> None:
        """Show the appropriate display based on parameters.

        Parameters
        ----------
        parts : Any, default: None
            Parts to show.
        update : bool, default: True
            Whether to update the display.
        spline : bool, default: False
            Whether to use splines.
        scope : prime.ScopeDefinition, default: None
            Scope of the parts.
        """
        plotter = PrimePlotter(use_trame=self.use_trame)
        plotter.add_model(self.model, scope=scope)
        plotter.show()



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class TopoData(CoreObject):
    """Topodata has all information about connectivity of nodes, edges, elements and more.

    Parameters
    ----------
    model : Model
        Server model to create TopoData object.
    id : int
        Id of the TopoData.
    object_id : int
        Object id of the TopoData.
    name : str
        Name of the TopoData.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize TopoData """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def get_geom_zonelets_of_topo_edges(self, topo_edges : Iterable[int]) -> Iterable[int]:
        """ Get the geometry edge zonelets for the provided topoedge ids.


        Parameters
        ----------
        topo_edges : Iterable[int]
            Ids of the topoedges.

        Returns
        -------
        Iterable[int]
            Return the geometry edge zonelet ids.


        Examples
        --------
        >>> geom_edge_zonelets = topo_data.get_geom_zonelets_of_topo_edges(topo_edges)

        """
        if not isinstance(topo_edges, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_edges'. Valid argument type is Iterable[int].")
        args = {"topo_edges" : topo_edges}
        command_name = "PrimeMesh::TopoData/GetGeomZoneletsOfTopoEdges"
        self._model._print_logs_before_command("get_geom_zonelets_of_topo_edges", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_geom_zonelets_of_topo_edges")
        return result

    def get_geom_zonelets_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the geometry face zonelets for the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topofaces.

        Returns
        -------
        Iterable[int]
            Return the geometry face zonelet ids.


        Examples
        --------
        >>> geom_face_zonelets = topo_data.get_geom_zonelets_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_faces'. Valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetGeomZoneletsOfTopoFaces"
        self._model._print_logs_before_command("get_geom_zonelets_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_geom_zonelets_of_topo_faces")
        return result

    def get_mesh_zonelets_of_topo_edges(self, topo_edges : Iterable[int]) -> Iterable[int]:
        """ Get the mesh edge zonelets for the provided topoedge ids.


        Parameters
        ----------
        topo_edges : Iterable[int]
            Ids of the topoedges.

        Returns
        -------
        Iterable[int]
            Return the mesh edge zonelet ids.


        Examples
        --------
        >>> mesh_edge_zonelets = topo_data.get_mesh_zonelets_of_topo_edges(topo_edges)

        """
        if not isinstance(topo_edges, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_edges'. Valid argument type is Iterable[int].")
        args = {"topo_edges" : topo_edges}
        command_name = "PrimeMesh::TopoData/GetMeshZoneletsOfTopoEdges"
        self._model._print_logs_before_command("get_mesh_zonelets_of_topo_edges", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_mesh_zonelets_of_topo_edges")
        return result

    def get_mesh_zonelets_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the mesh face zonelets for the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topofaces.

        Returns
        -------
        Iterable[int]
            Return the mesh face zonelet ids.


        Examples
        --------
        >>> mesh_face_zonelets = topo_data.get_mesh_zonelets_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_faces'. Valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetMeshZoneletsOfTopoFaces"
        self._model._print_logs_before_command("get_mesh_zonelets_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_mesh_zonelets_of_topo_faces")
        return result

    def get_topo_edges_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the topoedges of the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topoface.

        Returns
        -------
        Iterable[int]
            Returns the list of topoedge ids.


        Examples
        --------
        >>> topo_edges_of_topo_faces = topo_data.get_topo_edges_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_faces'. Valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetTopoEdgesOfTopoFaces"
        self._model._print_logs_before_command("get_topo_edges_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_topo_edges_of_topo_faces")
        return result

    def get_topo_volumes_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Gets the topovolumes of the given topofaces.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topofaces.

        Returns
        -------
        Iterable[int]
            Returns the list of topovolume ids.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> topovolumes = topo_data.get_topo_volumes_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_faces'. Valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetTopoVolumesOfTopoFaces"
        self._model._print_beta_api_warning("get_topo_volumes_of_topo_faces")
        self._model._print_logs_before_command("get_topo_volumes_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_topo_volumes_of_topo_faces")
        return result

    def get_adjacent_topo_faces_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the adjacent topofaces for the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topoface.

        Returns
        -------
        Iterable[int]
            Returns the list of adjacent topoface ids.


        Examples
        --------
        >>> topo_faces_of_topo_faces = topo_data.get_adjacent_topo_faces_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_faces'. Valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetAdjacentTopoFacesOfTopoFaces"
        self._model._print_logs_before_command("get_adjacent_topo_faces_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_adjacent_topo_faces_of_topo_faces")
        return result

    def get_adjacent_topo_edges_of_topo_edges(self, topo_edges : Iterable[int]) -> Iterable[int]:
        """ Get the adjacent topoedges for the provided topoedge ids.


        Parameters
        ----------
        topo_edges : Iterable[int]
            Ids of the topoedge.

        Returns
        -------
        Iterable[int]
            Returns the list of adjacent topoedge ids.


        Examples
        --------
        >>> topo_edges_of_topo_edges = topo_data.get_adjacent_topo_edges_of_topo_edges(topo_edges)

        """
        if not isinstance(topo_edges, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_edges'. Valid argument type is Iterable[int].")
        args = {"topo_edges" : topo_edges}
        command_name = "PrimeMesh::TopoData/GetAdjacentTopoEdgesOfTopoEdges"
        self._model._print_logs_before_command("get_adjacent_topo_edges_of_topo_edges", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_adjacent_topo_edges_of_topo_edges")
        return result

    def delete_mesh_on_topo_faces(self, topo_faces : Iterable[int], params : DeleteMeshParams) -> DeleteMeshResults:
        """ Delete mesh on the provided topofaces.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of topofaces.
        params : DeleteMeshParams
            Parameters to delete mesh on topofaces.

        Returns
        -------
        DeleteMeshResults
            Returns the DeleteMeshResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> params = prime.DeleteMeshParams(model = model)
        >>> result = topo_data.delete_mesh_on_topo_faces(top_faces, params)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for 'topo_faces'. Valid argument type is Iterable[int].")
        if not isinstance(params, DeleteMeshParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is DeleteMeshParams.")
        args = {"topo_faces" : topo_faces,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::TopoData/DeleteMeshOnTopoFaces"
        self._model._print_beta_api_warning("delete_mesh_on_topo_faces")
        self._model._print_logs_before_command("delete_mesh_on_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_mesh_on_topo_faces", DeleteMeshResults(model = self._model, json_data = result))
        return DeleteMeshResults(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of TopoData."""
        return self._id

    @property
    def name(self):
        """ Get the name of TopoData."""
        return self._name
